︠2e8103f8-67ae-4211-b24f-00d53e7a3053︠
# TO DO
#-Figure out plausible units for initial values and constants
#-Determine correct coefficient for arbitrary order RK (i.e. RK4 -> 1/6, RK5 -> ?)
#-Try to find a way to make RK1 = Euler method and RK2 = Midpoint method
#-Dynamic number of steps (# of oscillations, etc.)
#-International System units!

@interact
def solver(method_choice = selector(['Euler', 'Midpoint', 'Runge-Kutta'], default='Runge-Kutta', nrows=1, label='Method'),
           step_size = slider(vmin=0.0000001, vmax=0.1, step_size=0.0000001, default=1, label='Step size'),
           rk_order = input_box(default="4", label='Order of Runge-Kutta'),
           init_radius = input_box(default="1", label='Initial radius'),
           init_rad_vel = input_box(default="0", label='Initial radial velocity'),
           init_rad_acc = input_box(default="0", label='Initial radial acceleration'),
           f_bubble_pressure = input_box(default="1", label='Initial gas pressure inside bubble', width=10),
           f_v_pressure = input_box(default="1", label='Vapor pressure (p_v(T_inf))', width=10),
           f_i_pressure = input_box(default="1", label='Fluid pressure far away from cavitation', width=10),
           f_density = input_box(default="1", label='Liquid density', width=10),        #f_var indicates fluid variable
           f_viscosity = input_box(default="1", label='Liquid viscosity', width=10),
           f_surface_tension = input_box(default="1", label='Bubble surface tension', width=10),
           f_constant_k = input_box(default="1", label='Constant K', width=10)):
        
        h = step_size
        rk_order = int(rk_order)
        
        R_0 = int(init_radius)
        RV_0 = int(init_rad_vel)
        RA_0 = int(init_rad_acc)
        PB = int(f_bubble_pressure)
        PV = int(f_v_pressure)
        PI = int(f_i_pressure)
        D = int(f_density)
        V = int(f_viscosity)
        S = int(f_surface_tension)
        K = int(f_constant_k)
        
        #Function for linearized Rayleigh-Plesset
        def equation(r, rv):
            return sin(r)#return rv*((3*RV_0/r) + (4*V/r^2)) + (R_0^K)/(r^(3*K+1)) + (2*S/(3*r^3)) + ((PB-PV-PI)/D) - (3*RV_0^2)/(2*r)
        
        #Create some arrays to store values
        R_array = [[0,R_0]]
        RV_array = [[0,RV_0]]
        RA_array = [[0,equation(R_0,RV_0)]]
        slope_array = []
        extrema_array = []
        trend = True #True = ascending, False = descending
        osc_goal = 10
        osc_num = 0
        
        #and variables to use in computations
        r = R_0
        rv = RV_0
        ra = RA_0
        
        i = 1
        
        #BEGIN MAIN LOOP
        while i < 1000 or osc_goal < osc_num:
            if method_choice == 'Euler':
                #calculate the radius based on previous radius and velocity
                r = r + rv*h
                R_array.append([i*h,r])
                #calculate the velocity based on acceleration 'ra' and previous velocity
                rv = rv + ra*h
                RV_array.append([i*h,rv])
                #find acceleration based on linearized equation function
                ra = equation(r,rv)
                RA_array.append([i*h,ra])
            elif method_choice == 'Midpoint':
                r = r + rv*h
                rv = rv + ra*h
                ra = equation(r,rv)
                ra = equation(r+h/2, rv+ra/2)
                #append to arrays
                RA_array.append([i*h,ra])
                RV_array.append([i*h,rv])
                R_array.append([i*h,r])
            elif method_choice == 'Runge-Kutta':
                r = r + rv*h
                R_array.append([i*h,r])
                rv = rv + ra*h
                RV_array.append([i*h,rv])
                
                #calculation of 'ra' for arbitrary RK order with delta values stored in 'k_array'
                k_array = []
                k_array.append(h*equation(r,rv))
                for a in range(rk_order-2):
                    k_array.append(2*h*equation(r+h/2, rv+k_array[a]/2))
                k_array.append(h*equation(r+h,rv+k_array[rk_order-2]))
                
                ra = ra + sum(k_array)/6
                RA_array.append([i*h,ra])
            
            #calculate slope and detect extrema
            slope = (R_array[i][1] - R_array[i-1][1])/h
            slope_array.append(slope.n())  
            if len(extrema_array) == 0:
                trend = R_array[1][1] > R_array[0][1]    #on first iteration, set 'trend'
            elif R_array[i][1] > R_array[i-1][1] and trend == False:    #detect the new slope; if it changes, declare an extrema and switch 'trend'
                trend = True
                extrema_array.append(R_array[i-1])
            elif R_array[i][1] < R_array[i-1][1] and trend == True:    #same as above but for the opposite direction
                trend = False
                extrema_array.append(R_array[i-1])
            
            #check for low amounts of activity (overflow safeguard #1)
            avg_change = 0
            tot_change = 0    #not really the 'total' change - takes sign changes into account, unlike avg_change
            l = len(slope_array)
            if l > 5:
                for a in range(5):
                    avg_change += abs(slope_array[l-a-1] - slope_array[l-a-2])
                    tot_change += slope_array[l-a-1] - slope_array[l-a-2]
                avg_change /= 5
                tot_change /= 5
                if avg_change < 0.000005:
                    print "Computation interrupted due to lack of activity\n\t(Average change = %s)" % avg_change
                    break
            if avg_change > 100 and avg_change == abs(tot_change):
                print "Computation interrupted due to exponential behavior"
                break
            
            i += 1
        #END MAIN LOOP
        
        #Create a line that connects all of the points in r_array
        L = line([(R_array[j][0], R_array[j][1]) for j in range(len(R_array))], color=(1,0,0), legend_label="Radial position")
        show(L)
        
        #Same thing, RV_array
        M = line([(RV_array[j][0], RV_array[j][1]) for j in range(len(RV_array))], color=(0,1,0), legend_label="Radial velocity")
        show(M)
        
        #ctrl-c ctrl-v RA_array
        N = line([(RA_array[j][0], RA_array[j][1]) for j in range(len(RA_array))], color=(0,0,1), legend_label="Radial accel")
        show(N)
        show(L+M+N)
︡b06f1c8b-6473-4b25-94f5-106a6ecc97be︡{"html": "<!--notruncate-->\n        <div padding=6 id=\"div-interact-10\">\n          <table width=800px height=20px bgcolor=\"#c5c5c5\" cellpadding=15>\n            <tr>\n              <td bgcolor=\"#f9f9f9\" valign=top align=left>\n            <table>\n              <tr><td colspan=3><table><tr><td align=right><font color=\"black\">Method&nbsp;</font></td><td><table style=\"border:1px solid #dfdfdf; background-color:#efefef;\">\n<tr><td><button style=\"border-style:outset;\" value=\"0\" onclick=\"$('BUTTON', this.parentNode).css('border-style', 'outset'); $(this).css('border-style', 'inset'); interact(10, {variable: 'method_choice', adapt_number: 1, value: encode64(this.value)}, 1)\">Euler</button>\n<button style=\"border-style:outset;\" value=\"1\" onclick=\"$('BUTTON', this.parentNode).css('border-style', 'outset'); $(this).css('border-style', 'inset'); interact(10, {variable: 'method_choice', adapt_number: 1, value: encode64(this.value)}, 1)\">Midpoint</button>\n<button style=\"border-style:inset;\" value=\"2\" onclick=\"$('BUTTON', this.parentNode).css('border-style', 'outset'); $(this).css('border-style', 'inset'); interact(10, {variable: 'method_choice', adapt_number: 1, value: encode64(this.value)}, 1)\">Runge-Kutta</button>\n</td></tr></table></td>\n</tr><tr><td align=right><font color=\"black\">Step size&nbsp;</font></td><td>\n        <table>\n          <tr>\n            <td>\n              <div id=\"slider-step_size-10\" style=\"margin:0px; margin-left: 1.0em; margin-right: 1.0em; width: 15.0em;\"></div>\n            </td>\n            \n            <td>\n              <font color=\"black\" id=\"slider-step_size-10-lbl\"></font>\n            </td>\n          </tr>\n        </table><script>\n    (function() {\n        var values = [\"1.00000000000000e-7\",\"0.000200500000000004\",\"0.000400900000000009\",\"0.000601299999999952\",\"0.000801699999999849\",\"0.00100209999999977\",\"0.00120249999999989\",\"0.00140290000000000\",\"0.00160330000000011\",\"0.00180370000000023\",\"0.00200410000000023\",\"0.00220449999999991\",\"0.00240489999999959\",\"0.00260529999999927\",\"0.00280569999999895\",\"0.00300609999999862\",\"0.00320649999999830\",\"0.00340689999999798\",\"0.00360729999999766\",\"0.00380769999999734\",\"0.00400809999999746\",\"0.00420849999999801\",\"0.00440889999999856\",\"0.00460929999999911\",\"0.00480969999999965\",\"0.00501010000000020\",\"0.00521050000000075\",\"0.00541090000000130\",\"0.00561130000000185\",\"0.00581170000000239\",\"0.00601210000000294\",\"0.00621250000000349\",\"0.00641290000000404\",\"0.00661330000000459\",\"0.00681370000000513\",\"0.00701410000000568\",\"0.00721450000000623\",\"0.00741490000000678\",\"0.00761530000000733\",\"0.00781570000000785\",\"0.00801610000000666\",\"0.00821650000000547\",\"0.00841690000000428\",\"0.00861730000000309\",\"0.00881770000000190\",\"0.00901810000000071\",\"0.00921849999999952\",\"0.00941889999999833\",\"0.00961929999999714\",\"0.00981969999999594\",\"0.0100200999999948\",\"0.0102204999999936\",\"0.0104208999999924\",\"0.0106212999999912\",\"0.0108216999999900\",\"0.0110220999999888\",\"0.0112224999999876\",\"0.0114228999999864\",\"0.0116232999999852\",\"0.0118236999999840\",\"0.0120240999999829\",\"0.0122244999999817\",\"0.0124248999999805\",\"0.0126252999999793\",\"0.0128256999999781\",\"0.0130260999999769\",\"0.0132264999999757\",\"0.0134268999999745\",\"0.0136272999999733\",\"0.0138276999999721\",\"0.0140280999999710\",\"0.0142284999999698\",\"0.0144288999999686\",\"0.0146292999999674\",\"0.0148296999999662\",\"0.0150300999999650\",\"0.0152304999999638\",\"0.0154308999999626\",\"0.0156312999999614\",\"0.0158316999999602\",\"0.0160320999999591\",\"0.0162324999999579\",\"0.0164328999999567\",\"0.0166332999999555\",\"0.0168336999999543\",\"0.0170340999999531\",\"0.0172344999999519\",\"0.0174348999999507\",\"0.0176352999999495\",\"0.0178356999999483\",\"0.0180360999999471\",\"0.0182364999999460\",\"0.0184368999999448\",\"0.0186372999999436\",\"0.0188376999999424\",\"0.0190380999999412\",\"0.0192384999999400\",\"0.0194388999999388\",\"0.0196392999999376\",\"0.0198396999999364\",\"0.0200400999999352\",\"0.0202404999999341\",\"0.0204408999999329\",\"0.0206412999999317\",\"0.0208416999999305\",\"0.0210420999999293\",\"0.0212424999999281\",\"0.0214428999999269\",\"0.0216432999999257\",\"0.0218436999999245\",\"0.0220440999999233\",\"0.0222444999999222\",\"0.0224448999999210\",\"0.0226452999999198\",\"0.0228456999999186\",\"0.0230460999999174\",\"0.0232464999999162\",\"0.0234468999999150\",\"0.0236472999999138\",\"0.0238476999999126\",\"0.0240480999999114\",\"0.0242484999999103\",\"0.0244488999999091\",\"0.0246492999999079\",\"0.0248496999999067\",\"0.0250500999999055\",\"0.0252504999999043\",\"0.0254508999999031\",\"0.0256512999999019\",\"0.0258516999999007\",\"0.0260520999998995\",\"0.0262524999998984\",\"0.0264528999998972\",\"0.0266532999998960\",\"0.0268536999998948\",\"0.0270540999998936\",\"0.0272544999998924\",\"0.0274548999998912\",\"0.0276552999998900\",\"0.0278556999998888\",\"0.0280560999998876\",\"0.0282564999998865\",\"0.0284568999998853\",\"0.0286572999998841\",\"0.0288576999998829\",\"0.0290580999998817\",\"0.0292584999998805\",\"0.0294588999998793\",\"0.0296592999998781\",\"0.0298596999998769\",\"0.0300600999998757\",\"0.0302604999998746\",\"0.0304608999998734\",\"0.0306612999998722\",\"0.0308616999998710\",\"0.0310620999998698\",\"0.0312624999998690\",\"0.0314628999998748\",\"0.0316632999998806\",\"0.0318636999998863\",\"0.0320640999998921\",\"0.0322644999998978\",\"0.0324648999999036\",\"0.0326652999999094\",\"0.0328656999999151\",\"0.0330660999999209\",\"0.0332664999999267\",\"0.0334669999999324\",\"0.0336673999999382\",\"0.0338677999999440\",\"0.0340681999999497\",\"0.0342685999999555\",\"0.0344689999999612\",\"0.0346693999999670\",\"0.0348697999999728\",\"0.0350701999999785\",\"0.0352705999999843\",\"0.0354709999999901\",\"0.0356713999999958\",\"0.0358718000000016\",\"0.0360722000000073\",\"0.0362726000000131\",\"0.0364730000000189\",\"0.0366734000000246\",\"0.0368738000000304\",\"0.0370742000000362\",\"0.0372746000000419\",\"0.0374750000000477\",\"0.0376754000000534\",\"0.0378758000000592\",\"0.0380762000000650\",\"0.0382766000000707\",\"0.0384770000000765\",\"0.0386774000000823\",\"0.0388778000000880\",\"0.0390782000000938\",\"0.0392786000000995\",\"0.0394790000001053\",\"0.0396794000001111\",\"0.0398798000001168\",\"0.0400802000001226\",\"0.0402806000001284\",\"0.0404810000001341\",\"0.0406814000001399\",\"0.0408818000001456\",\"0.0410822000001514\",\"0.0412826000001572\",\"0.0414830000001629\",\"0.0416834000001687\",\"0.0418838000001745\",\"0.0420842000001802\",\"0.0422846000001860\",\"0.0424850000001917\",\"0.0426854000001975\",\"0.0428858000002033\",\"0.0430862000002090\",\"0.0432866000002148\",\"0.0434870000002206\",\"0.0436874000002263\",\"0.0438878000002321\",\"0.0440882000002378\",\"0.0442886000002436\",\"0.0444890000002494\",\"0.0446894000002551\",\"0.0448898000002609\",\"0.0450902000002667\",\"0.0452906000002724\",\"0.0454910000002782\",\"0.0456914000002839\",\"0.0458918000002897\",\"0.0460922000002955\",\"0.0462926000003012\",\"0.0464930000003070\",\"0.0466934000003128\",\"0.0468938000003185\",\"0.0470942000003243\",\"0.0472946000003300\",\"0.0474950000003358\",\"0.0476954000003416\",\"0.0478958000003473\",\"0.0480962000003531\",\"0.0482966000003589\",\"0.0484970000003646\",\"0.0486974000003704\",\"0.0488978000003761\",\"0.0490982000003819\",\"0.0492986000003877\",\"0.0494990000003934\",\"0.0496994000003992\",\"0.0498998000004050\",\"0.0501002000004107\",\"0.0503006000004165\",\"0.0505010000004222\",\"0.0507014000004280\",\"0.0509018000004338\",\"0.0511022000004395\",\"0.0513026000004453\",\"0.0515030000004511\",\"0.0517034000004568\",\"0.0519038000004626\",\"0.0521042000004684\",\"0.0523046000004741\",\"0.0525050000004799\",\"0.0527054000004856\",\"0.0529058000004914\",\"0.0531062000004972\",\"0.0533066000005029\",\"0.0535070000005087\",\"0.0537074000005145\",\"0.0539078000005202\",\"0.0541082000005260\",\"0.0543086000005317\",\"0.0545090000005375\",\"0.0547094000005433\",\"0.0549098000005490\",\"0.0551102000005548\",\"0.0553106000005606\",\"0.0555110000005663\",\"0.0557114000005721\",\"0.0559118000005778\",\"0.0561122000005836\",\"0.0563126000005894\",\"0.0565130000005951\",\"0.0567134000006009\",\"0.0569138000006067\",\"0.0571142000006124\",\"0.0573146000006182\",\"0.0575150000006239\",\"0.0577154000006297\",\"0.0579158000006355\",\"0.0581162000006412\",\"0.0583166000006470\",\"0.0585170000006528\",\"0.0587174000006585\",\"0.0589178000006643\",\"0.0591182000006700\",\"0.0593186000006758\",\"0.0595190000006816\",\"0.0597194000006873\",\"0.0599198000006931\",\"0.0601202000006989\",\"0.0603206000007046\",\"0.0605210000007104\",\"0.0607214000007161\",\"0.0609218000007219\",\"0.0611222000007277\",\"0.0613226000007334\",\"0.0615230000007392\",\"0.0617234000007450\",\"0.0619238000007507\",\"0.0621242000007565\",\"0.0623246000007622\",\"0.0625250000007680\",\"0.0627254000007738\",\"0.0629258000007795\",\"0.0631262000007853\",\"0.0633266000007911\",\"0.0635270000007968\",\"0.0637274000008026\",\"0.0639278000008083\",\"0.0641282000008141\",\"0.0643286000008199\",\"0.0645290000008256\",\"0.0647294000008314\",\"0.0649298000008372\",\"0.0651302000008429\",\"0.0653306000008487\",\"0.0655310000008544\",\"0.0657314000008602\",\"0.0659318000008660\",\"0.0661322000008717\",\"0.0663326000008775\",\"0.0665330000008833\",\"0.0667335000008890\",\"0.0669339000008948\",\"0.0671343000009006\",\"0.0673347000009063\",\"0.0675351000009121\",\"0.0677355000009178\",\"0.0679359000009236\",\"0.0681363000009294\",\"0.0683367000009351\",\"0.0685371000009409\",\"0.0687375000009467\",\"0.0689379000009524\",\"0.0691383000009582\",\"0.0693387000009639\",\"0.0695391000009697\",\"0.0697395000009755\",\"0.0699399000009812\",\"0.0701403000009870\",\"0.0703407000009928\",\"0.0705411000009985\",\"0.0707415000010043\",\"0.0709419000010100\",\"0.0711423000010158\",\"0.0713427000010216\",\"0.0715431000010273\",\"0.0717435000010331\",\"0.0719439000010389\",\"0.0721443000010446\",\"0.0723447000010504\",\"0.0725451000010561\",\"0.0727455000010619\",\"0.0729459000010677\",\"0.0731463000010734\",\"0.0733467000010792\",\"0.0735471000010850\",\"0.0737475000010907\",\"0.0739479000010965\",\"0.0741483000011022\",\"0.0743487000011080\",\"0.0745491000011138\",\"0.0747495000011195\",\"0.0749499000011253\",\"0.0751503000011311\",\"0.0753507000011368\",\"0.0755511000011426\",\"0.0757515000011483\",\"0.0759519000011541\",\"0.0761523000011599\",\"0.0763527000011656\",\"0.0765531000011714\",\"0.0767535000011772\",\"0.0769539000011829\",\"0.0771543000011887\",\"0.0773547000011944\",\"0.0775551000012002\",\"0.0777555000012060\",\"0.0779559000012117\",\"0.0781563000012175\",\"0.0783567000012233\",\"0.0785571000012290\",\"0.0787575000012348\",\"0.0789579000012405\",\"0.0791583000012463\",\"0.0793587000012521\",\"0.0795591000012578\",\"0.0797595000012636\",\"0.0799599000012694\",\"0.0801603000012751\",\"0.0803607000012809\",\"0.0805611000012866\",\"0.0807615000012924\",\"0.0809619000012982\",\"0.0811623000013039\",\"0.0813627000013097\",\"0.0815631000013155\",\"0.0817635000013212\",\"0.0819639000013270\",\"0.0821643000013327\",\"0.0823647000013385\",\"0.0825651000013443\",\"0.0827655000013500\",\"0.0829659000013558\",\"0.0831663000013616\",\"0.0833667000013673\",\"0.0835671000013731\",\"0.0837675000013789\",\"0.0839679000013846\",\"0.0841683000013904\",\"0.0843687000013961\",\"0.0845691000014019\",\"0.0847695000014077\",\"0.0849699000014134\",\"0.0851703000014192\",\"0.0853707000014250\",\"0.0855711000014307\",\"0.0857715000014365\",\"0.0859719000014422\",\"0.0861723000014480\",\"0.0863727000014538\",\"0.0865731000014595\",\"0.0867735000014653\",\"0.0869739000014711\",\"0.0871743000014768\",\"0.0873747000014826\",\"0.0875751000014883\",\"0.0877755000014941\",\"0.0879759000014999\",\"0.0881763000015056\",\"0.0883767000015114\",\"0.0885771000015172\",\"0.0887775000015229\",\"0.0889779000015287\",\"0.0891783000015344\",\"0.0893787000015402\",\"0.0895791000015460\",\"0.0897795000015517\",\"0.0899799000015575\",\"0.0901803000015633\",\"0.0903807000015690\",\"0.0905811000015748\",\"0.0907815000015805\",\"0.0909819000015863\",\"0.0911823000015921\",\"0.0913827000015978\",\"0.0915831000016036\",\"0.0917835000016094\",\"0.0919839000016151\",\"0.0921843000016209\",\"0.0923847000016266\",\"0.0925851000016324\",\"0.0927855000016382\",\"0.0929859000016439\",\"0.0931863000016497\",\"0.0933867000016555\",\"0.0935871000016612\",\"0.0937875000016670\",\"0.0939879000016727\",\"0.0941883000016785\",\"0.0943887000016843\",\"0.0945891000016900\",\"0.0947895000016958\",\"0.0949899000017016\",\"0.0951903000017073\",\"0.0953907000017131\",\"0.0955911000017188\",\"0.0957915000017246\",\"0.0959919000017304\",\"0.0961923000017361\",\"0.0963927000017419\",\"0.0965931000017477\",\"0.0967935000017534\",\"0.0969939000017592\",\"0.0971943000017649\",\"0.0973947000017707\",\"0.0975951000017765\",\"0.0977955000017822\",\"0.0979959000017880\",\"0.0981963000017938\",\"0.0983967000017995\",\"0.0985971000018053\",\"0.0987975000018110\",\"0.0989979000018168\",\"0.0991983000018226\",\"0.0993987000018283\",\"0.0995991000018341\",\"0.0997995000018399\",\"0.100000000000000\"];\n        setTimeout(function() {\n            $('#slider-step_size-10').slider({\n                step: 1,\n                min: 0,\n                max: 499,\n                value: 499,\n                change: function (e, ui) {\n                    var position = ui.value;\n                    if (values != null) {\n                        $('#slider-step_size-10-lbl').text(values[position]);\n                        interact(10, {variable: 'step_size', adapt_number: 2, value: encode64(position)}, 1);\n                    }\n                },\n                slide: function (e, ui) {\n                    if (values != null) {\n                        $('#slider-step_size-10-lbl').text(values[ui.value]);\n                    }\n                }\n            });\n            if (values != null) {\n                $('#slider-step_size-10-lbl').text(values[$('#slider-step_size-10').slider('value')]);\n            }\n        }, 1);\n    })();\n    </script></td>\n</tr><tr><td align=right><font color=\"black\">Order of Runge-Kutta&nbsp;</font></td><td><input type=\"text\" value=\"4\" size=80 onchange=\"interact(10, {variable: 'rk_order', adapt_number: 3, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Initial radius&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=80 onchange=\"interact(10, {variable: 'init_radius', adapt_number: 4, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Initial radial velocity&nbsp;</font></td><td><input type=\"text\" value=\"0\" size=80 onchange=\"interact(10, {variable: 'init_rad_vel', adapt_number: 5, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Initial radial acceleration&nbsp;</font></td><td><input type=\"text\" value=\"0\" size=80 onchange=\"interact(10, {variable: 'init_rad_acc', adapt_number: 6, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Initial gas pressure inside bubble&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_bubble_pressure', adapt_number: 7, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Vapor pressure (p_v(T_inf))&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_v_pressure', adapt_number: 8, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Fluid pressure far away from cavitation&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_i_pressure', adapt_number: 9, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Liquid density&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_density', adapt_number: 10, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Liquid viscosity&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_viscosity', adapt_number: 11, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Bubble surface tension&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_surface_tension', adapt_number: 12, value: encode64(this.value)}, 1)\"></input></td>\n</tr><tr><td align=right><font color=\"black\">Constant K&nbsp;</font></td><td><input type=\"text\" value=\"1\" size=10 onchange=\"interact(10, {variable: 'f_constant_k', adapt_number: 13, value: encode64(this.value)}, 1)\"></input></td>\n</tr></table></td></tr>\n              <tr><td></td><td style='width: 100%;'>\n        <div id=\"cell-interact-10\"><?__SAGE__START>\n          <table border=0 bgcolor=\"white\" width=100%>\n            <tr>\n              <td bgcolor=\"white\" align=left valign=top>\n                <pre><?__SAGE__TEXT></pre>\n              </td>\n            </tr>\n            <tr>\n              <td align=left valign=top><?__SAGE__HTML></td>\n            </tr>\n          </table><?__SAGE__END>\n        </div></td><td></td></tr>\n              <tr><td colspan=3></td></tr>\n            </table></td>\n            </tr>\n          </table>\n        </div>"}︡
︠0fe2665c-2696-44fc-8603-874a87c9cdb3︠
#Implement this code into the main bubble simulation
#-Convert main simulations to 'while' loops
#-Add in the overflow/nonactivity detector
#-Create function to evaluate an extreme point given a slice of the array
#-HAVE THE RANGES FOR OVERFLOW DETECTION SCALE WITH STEP SIZE

#Finish this code
#-Account for a triple peak with middle peak being lowest
#-Implement run-time extrema evaluation (see below)

array = [[0,0]]
slope_array = []
extrema_array = []
n = 0.1 #step size
trend = True #True = ascending, False = descending
osc_goal = 10
osc_num = 0

i = 1

while osc_num < osc_goal:
    #calculate new x, y, and slope; add them to respective arrays
    x = i*n
    y = sin(x) + 0.1*sin(10*x)# + 0.2*sin(20*x)# + 0.3*sin(30*x)
    array.append([x,y])
    slope = (array[i][1] - array[i-1][1])/(4*pi/n)
    slope_array.append(slope.n())
    
    #detect extrema
    if i == 0:
        trend = y > array[0][1]    #on first iteration, set 'trend'
    elif array[i][1] > array[i-1][1] and trend == False:    #detect the new slope; if it changes, declare an extrema and switch 'trend'
        trend = True
        extrema_array.append(array[i-1])
    elif array[i][1] < array[i-1][1] and trend == True:    #same as above but for the opposite direction
        trend = False
        extrema_array.append(array[i-1])
    
    #guess at a typical length of oscillation
    #possible things to look for: distance traveled in y direction 
    sorted(extrema_array)
    
    #check for low amounts of activity or exponential behavior
    avg_change = 0
    tot_change = 0    #not really the 'total' change - takes sign changes into account, unlike avg_change
    l = len(slope_array)
    if l > 5:
        for a in range(5):
            avg_change += abs(slope_array[l-a-1] - slope_array[l-a-2])
            tot_change += slope_array[l-a-1] - slope_array[l-a-2]
        avg_change /= 5
        tot_change /= 5
        if avg_change < 0.000005:
            print "Computation interrupted due to lack of activity\n\t(Average change = %s)" % avg_change
            break
        if avg_change > 100 and avg_change == abs(tot_change):
            print "Computation interrupted due to exponential behavior"
            break
     
    i += 1

#Convert this into a run-time evaluation of extrema
#REMEMBER: You can't have 2 consecutive maxima - must be a minimum in between!
#Think about comparing y-distance of extrema to known min and max

    extrema_array_2 = [] #this will hold all the "significant" extrema
    prev_value = ""
    for a in range(len(extrema_array)-2):
        if a < 2: continue
    
        search_point = list(extrema_array[a])
        prev_point = list(extrema_array[a-1])
        prev_ext = list(extrema_array[a-2])
        next_ext = list(extrema_array[a+2])
    
        if (search_point[1] > prev_point[1] and search_point[1] >= prev_ext[1] and search_point[1] >= next_ext[1]):
            search_point.append("max")
            if (len(extrema_array_2) != 0):
                if (extrema_array_2[len(extrema_array_2)-1][2] == "min"):
                    extrema_array_2.append(search_point)
                elif (search_point[1].n() > extrema_array_2[len(extrema_array_2)-1][1].n()):
                    extrema_array_2.pop()
                    extrema_array_2.append(search_point)
                elif (search_point[1].n() < extrema_array_2[len(extrema_array_2)-1][1].n()):
                    pass
            else:
                extrema_array_2.append(search_point)
        elif (search_point[1] < prev_point[1] and search_point[1] <= prev_ext[1] and search_point[1] <= next_ext[1]):
            search_point.append("min")
            if (len(extrema_array_2) > 0):
                if (extrema_array_2[len(extrema_array_2)-1][2] == "max"):
                    extrema_array_2.append(search_point)
                elif (search_point[1].n() < extrema_array_2[len(extrema_array_2)-1][1].n()):
                    extrema_array_2.pop()
                    extrema_array_2.append(search_point)
                elif (search_point[1].n() > extrema_array_2[len(extrema_array_2)-1][1].n()):
                    pass
            else:
                extrema_array_2.append(search_point)
    
    osc_num = len(extrema_array_2)/2

print len(extrema_array_2)
graph = line([(array[j][0].n(), array[j][1].n()) for j in range(len(array))])
points = point([(extrema_array_2[j][0].n(), extrema_array_2[j][1].n()) for j in    range(len(extrema_array_2))], rgbcolor=(1,0,0), pointsize=18)
points2 = point([(extrema_array[j][0].n(), extrema_array[j][1].n()) for j in range(len(extrema_array))], rgbcolor=(0,1,0), pointsize=18)
show(points2 + points + graph)
︡d491edd1-ec4d-4037-9ce7-d58fa84ef743︡{"stdout": "20"}︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠a59b30c4-91ed-4659-865f-66c500626d12︠
#DETERMINING 'SIGNIFICANT' EXTREMA
#-An extrema is significant if the two surrounding extrema of the same type (minimum/maximum) are both higher or both lower, respectively
#-An extrema is not significant if its neighboring extrema is of the same type and of greater magnitude
#-An extrema is *probably* not significant if it the change in Y between it and the neighboring extrema of the opposite type is significantly less than that of the furthest separated pair 
#    -Thought: Margin of acceptability defined by: (distance between absolute min and max) - (distance between furthest apart extrema)
︡d453aaf0-1ae2-4647-bc8a-cc96d0495bff︡︡
︠fb95ab67-50c5-40d3-a8e5-4103e4b04bd2︠
def runge_kutta_two(a,b,N,alpha):
    #create functions we will need
    f = lambda x,y: 1.0 + (y/x) + (y/x)^2    #this is the first equation
    g = lambda x: x*tan(ln(x))               # this is the exact solution for the IVP
    #continuing with page 279
    h = (b-a)/N
    t = a
    omega = alpha
    #make a list to store values
    w = []
    w.append((0,omega, numerical_approx(abs(g(1)-omega))))    # this stores y(1) = 0 & actual error
    for i in range(1,N):
        omega = omega + h * f(t + (h/2), omega + (h/t)*f(t,omega))        # compute omega_i
        w.append( ( i, omega, numerical_approx(abs(g(1)-omega))) )
        #store values
        t = a + i * h     # compute t_i
        return w

print runge_kutta_two(1,100,100,0)

#a = start point
#b = endpoint
#N = number of steps
#alpha = 

#h = step size
︡4dc3e877-b7ee-4344-b621-8e10b9a8a9cd︡︡
︠a92daace-7995-4d16-b431-fd92c7c306d7︠
if method_choice == 'Euler':
            for i in range(math.ceil(n/h)):
                #calculate the radius based on previous radius and velocity
                r = r + rv*h
                R_array.append([i*h,r])
                #calculate the velocity based on acceleration 'ra' and previous velocity
                rv = rv + ra*h
                RV_array.append([i*h,rv])
                #find acceleration based on linearized equation function
                ra = equation(r,rv)
                RA_array.append([i*h,ra])
        elif method_choice == 'Midpoint':
            for i in range(math.ceil(n/h)):
                r = r + rv*h
                rv = rv + ra*h
                ra = equation(r,rv)
                ra = equation(r+h/2, rv+ra/2)
                #append to arrays
                RA_array.append([i*h,ra])
                RV_array.append([i*h,rv])
                R_array.append([i*h,r])
        elif method_choice == 'Runge-Kutta':
            for i in range(math.ceil(n/h)):
                r = r + rv*h
                R_array.append([i*h,r])
                rv = rv + ra*h
                RV_array.append([i*h,rv])
                
                #calculation of 'ra' for arbitrary RK order with delta values stored in 'k_array'
                k_array = []
                k_array.append(h*equation(r,rv))
                for a in range(rk_order-2):
                    k_array.append(2*h*equation(r+h/2, rv+k_array[a]/2))
                k_array.append(h*equation(r+h,rv+k_array[rk_order-2]))
                
                ra = ra + sum(k_array)/6
                RA_array.append([i*h,ra])
︡6856d4f0-7b2d-4f8c-bd32-673106bb68d7︡{"stderr": "Traceback (most recent call last):                    rv = rv + ra*h\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpL_gBWK/___code___.py\", line 14\n    elif method_choice == 'Midpoint':\n                                    ^\nIndentationError: unindent does not match any outer indentation level"}︡
